@@ -9,7 +9,7 @@ Peak format:
     'rtime', 'peak_area', 'goodness_fitting', 'snr', 'cSelectivity',
 }
 '''
-
+import multiprocessing as mp
 import numpy as np
 from scipy.signal import find_peaks 
 from scipy.optimize import curve_fit 
@@ -53,12 +53,51 @@ def evaluate_gaussian_peak(mass_track, peak):
 # peak detection
 # -----------------------------------------------------------------------------
 
-def deep_detect_elution_peaks( mass_track, max_rt_number,
-                min_peak_height=10000, min_fwhm=3, min_prominence_threshold=5000, wlen=50, 
-                snr=2, min_prominence_ratio=0.1,
-                iteration=True):
+
+def batch_deep_detect_elution_peaks(list_mass_tracks, max_rt_number, parameters):
+    with mp.Manager() as manager:
+        shared_list = manager.list()
+        iters = iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters, shared_list)
+        with mp.Pool( parameters['multicores'] ) as pool:
+            pool.starmap( deep_detect_elution_peaks, iters )
+
+        FeatureList = list(shared_list)
+    return FeatureList
+
+def iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters, shared_list):
+    iters = []
+    min_peak_height = parameters['min_peak_height']
+    min_prominence_threshold = parameters['min_prominence_threshold']
+    min_fwhm = round( 0.5 * parameters['min_timepoints'] )
+    snr = parameters['signal_noise_ratio']
+    wlen = 50
+    min_prominence_ratio = 0.1
+    iteration = False
+    for mass_track in list_mass_tracks:
+        if max(mass_track['intensity']) > min_peak_height:
+            iters.append(
+                (mass_track, max_rt_number, min_peak_height, min_fwhm, min_prominence_threshold,
+                wlen, snr, min_prominence_ratio, iteration, shared_list)
+            )
+    return iters
+
+
+def deep_detect_elution_peaks( mass_track, max_rt_number, 
+                min_peak_height, min_fwhm, min_prominence_threshold,
+                wlen, snr, min_prominence_ratio, iteration,
+                shared_list):
     '''
     Optimized peak detection on a mass track. 
+
+    min_peak_height=10000, min_fwhm=3, min_prominence_threshold=5000, wlen=50, 
+                snr=2, min_prominence_ratio=0.1,
+                iteration=True, 
+
+    Can add further stats based ROI selection to optimize -
+                - not requiring continuous track, but ROIs will be continuous and used for peak detection 
+
+
+
     Local maxima is used in find_peak, but prominence and cSelectivity are used to control detection,
     in step-wise optimization. 
     Peak shape (Gaussian fitting goodness > 0.8) is required for noisy data or iterated detection.
@@ -181,13 +220,14 @@ def deep_detect_elution_peaks( mass_track, max_rt_number,
         if __noise_level__.size > 0:
             __noise_level__ = __noise_level__.mean()
         else:
-            __noise_level__ = __list_intensity[_peak_datapoints].min()
+            __noise_level__ = max(__list_intensity[_peak_datapoints].min(), 1)
         for _jpeak in list_json_peaks:
             _jpeak['snr'] = int( min(_jpeak['height'], 99999999) / __noise_level__)         # cap upper limit and avoid INF
             if _jpeak['snr'] > snr:
                 list_peaks.append(_jpeak)
 
-    return list_peaks
+    shared_list += list_peaks
+
 
 
 def convert_peak_json__( ii, mass_track, peaks, properties, cSelectivity=None):
@@ -213,7 +253,8 @@ def convert_peak_json__( ii, mass_track, peaks, properties, cSelectivity=None):
 
 def __peaks_cSelectivity__(__list_intensity, peaks, properties):
     '''
-    peaks, properties as from find_peaks; __list_intensity is np.array of the mass_track.
+    peaks, properties as from find_peaks; 
+    __list_intensity is np.array of the mass_track.
 
     Chromatographic peak selectivity (cSelectivity) is defined by 
     the ratio of the data points in all peaks above 1/2 this peak height and all data points above 1/2 this peak height.
@@ -229,13 +270,16 @@ def __peaks_cSelectivity__(__list_intensity, peaks, properties):
     list_cSelectivity = []
     for ii in range(peaks.size):
         _threshold = 0.5 * properties['peak_heights'][ii]
-        list_cSelectivity.append(
-            _peak_datapoints[_peak_datapoints > _threshold].size / __list_intensity[__list_intensity > _threshold].size
-        )
+        _peak_datapoints_level = _peak_datapoints[_peak_datapoints > _threshold].size
+        _background_level = __list_intensity[__list_intensity > _threshold].size
+        if _background_level >= _peak_datapoints_level > 0:                  # to rule out artifact from smooth_lowess
+            list_cSelectivity.append(_peak_datapoints_level / _background_level)
+        else:
+            list_cSelectivity.append( 0 )
+            
     return list_cSelectivity
 
 
-
 def detect_elution_peaks( mass_track, 
             min_peak_height=10000, min_fwhm=3, min_prominence_threshold=5000, wlen=50 ):
     '''Standard peak detection, to be used for exploration.
